@@ -34,6 +34,7 @@ dimmer::dimmer(QWidget *parent) :
 	iAnim = 0;
 	iAnimDir = 1;
 	iAnimY = this->rect().height()-this->rect().height()/4;
+	iProgress = -1;
 
 	iTimer = new QTimer(this);
 	connect(iTimer,SIGNAL(timeout()),this,SLOT(timerEvent()));
@@ -50,15 +51,17 @@ void dimmer::paintEvent(QPaintEvent *)
 	dim.setAlpha(iAlpha);
 	painter.setPen(dim);
 	painter.setBrush(dim);
-	painter.drawRect(this->rect());
+	painter.drawRect(rect());
 
 	painter.setBrush( QApplication::palette().color(QPalette::Highlight) );
 
 	int step = 15;
 	int animMax = 10;
 
-	painter.drawEllipse(QPoint(this->rect().left()+(this->rect().width()/2-(animMax*step)/2)+iAnim*step,
-							   iAnimY), 10, 10);
+	if( iProgress == -1 ) {
+		painter.drawEllipse(QPoint(rect().left()+(rect().width()/2-(animMax*step)/2)+iAnim*step,
+							iAnimY), 10, 10);
+	}
 
 	iAnim += iAnimDir;
 	if( iAnim>animMax )
@@ -66,6 +69,17 @@ void dimmer::paintEvent(QPaintEvent *)
 	if( iAnim==-1 )
 		iAnimDir=1;
 
+	if( iProgress >= 0 ) {
+		if( iProgress>100 )
+			iProgress = 100;
+
+		painter.setBrush( QApplication::palette().color(QPalette::Window) );
+		painter.drawRect(rect().left()+30, rect().bottom()-30, rect().right()-rect().left()-60, 20 );
+		painter.setBrush( QApplication::palette().color(QPalette::Highlight) );
+		painter.setPen(Qt::NoPen);
+		int pw = ( rect().right()-rect().left()-60 ) * iProgress / 100;
+		painter.drawRect(rect().left()+30, rect().bottom()-30, pw, 20 );
+	}
 }
 
 void dimmer::timerEvent()
@@ -88,14 +102,28 @@ void dimmer::orientationChanged()
 
 void dimmer::dim(QString title, QString message)
 {
+	iTitle = title;
 	iBusy = true;
-	iLabel->setText("<font color=\"white\"><b><u>" + title + "</u></b><br><br>" + message + "</font>");
-	show();
+	iLabel->setText("<font color=\"white\"><b><u>" + iTitle + "</u></b><br><br>" + message + "</font>");
+	iProgress = -1;
 	iAnim = 0;
 	iAnimDir = 1;
+
+	show();
 	iTimer->start(250);
 }
 
+void dimmer::updateText(QString message)
+{
+	iLabel->setText("<font color=\"white\"><b><u>" + iTitle + "</u></b><br><br>" + message + "</font>");
+}
+
+void dimmer::setProgress(int p_)
+{
+	iProgress = p_;
+	repaint(0,rect().bottom()-30,rect().width(),30);
+}
+
 void dimmer::undim()
 {
 	iBusy = false;
