@@ -21,6 +21,10 @@
 #include "mainwindow.h"
 #include "confirmdialog.h"
 
+#ifdef Q_WS_MAEMO_5
+#include <unistd.h>
+#endif
+
 int main(int argc, char *argv[])
 {
     QApplication a(argc, argv);
@@ -35,7 +39,7 @@ int main(int argc, char *argv[])
 	uid_t userUID = getuid();
 	if( userUID != 0 ) {
 		ConfirmDialog d(false, &w);
-		d.setText("Warning", "You are not running the application as root. It might not work as intended.");
+		d.setText("Warning", "You are not running the application as root. It won't' work as intended.");
 		d.exec();
 	}
 #endif
