@@ -21,7 +21,8 @@
 #define PACKAGE_H
 
 #include <QtCore>
-class AptInterface;
+class AAptInterface;
+class Repository;
 
 class Package
 {
@@ -31,63 +32,72 @@ public:
 	// the order must match status filter strings
 	enum packageStatus { PkgStatUnknown, PkgStatNotInstalled, PkgStatUpgradeable, PkgStatInstalled };
 
-	Package(QString name_, AptInterface* apt_);
+	Package(QString name_, AAptInterface* apt_);
 	~Package();
+	void setName(QString n_) { iName=n_; }
 	void setMaemoDisplayName(QString n_) { iMaemoDisplayName=n_; }
 	void setInstalled(bool i_) { iIsInstalled=i_; }
-	void setLocal(bool l_) { iIsLocal=l_; }
 	void setMarkedForOperation(operation op_);
-	void setVersionInstalled(QString v_) { iVersionInstalled=v_; }
-	void setVersionAvailable(QString v_) { iVersionAvailable=v_; }
+	void setVersion(QString v_) { iVersion=v_; }
 	void setDescShort(QString d_) { iDescriptionShort=d_; }
 	void appendDescLong(QString d_) { iDescriptionLong.append(d_); }
 	void setSection(QString s_) { iSection=s_; }
 	void setSize(int i_) { iSize=i_; }
 	void setInstalledSize(int i_) { iInstalledSize=i_; }
 	void appendIconData(QByteArray d_) { iIconData.append(d_); }
+	void setFileNameFull(QString f_) { iFileNameFull=f_; }
 	void updateStatus();
+	void addRepository(Repository* r_) { iRepositories.append(r_); }
+	void setDate(QDateTime d_) { iDate=d_; }
 
 	void convertIcon();
-	void printDebugInfo();
 
 	QString name() { return iName; }
 	QString maemoDisplayName() { return iMaemoDisplayName; }
+	QString displayName();
 	bool isInstalled() { return iIsInstalled; }
-	bool isLocal() { return iIsLocal; }
 	bool isMarkedForOperation() { return iMarkedForOperation; }
 	operation markedOperation() { return iMarkedOperation; }
-	QString versionInstalled() { return iVersionInstalled; }
-	QString versionAvailable() { return iVersionAvailable; }
+	QString version() { return iVersion; }
 	QString descShort() { return iDescriptionShort; }
 	QString descLong() { return iDescriptionLong; }
 	QString section() { return iSection; }
 	int size() { return iSize; }
-	int installedSize() { return iSize; }
-	QPixmap* icon() { return &iIcon; }
+	int installedSize() { return iInstalledSize; }
+	QPixmap* icon() { return iIcon; }
 	bool hasIconData();
 	bool isUpgradeable();
-	packageStatus status() { return iPkgStatus; }
+	QString upgradeableVersion();
+	Package* availablePackage();
+	packageStatus status();
+	QString fileName() { return iFileNameFull.mid( iFileNameFull.lastIndexOf('/')+1 ); }
+	QString fileNameFull() { return iFileNameFull; }
+	QDateTime date() { return iDate; }
+	QList<Repository*> repositories() { return iRepositories; }
+
+	static bool versionCompare(QString isNewer, QString compare);
 
 private:
-	AptInterface* iAptInterface;
+	AAptInterface* iAptInterface;
 
 	QString iName;
 	QString iMaemoDisplayName;
 	bool iIsInstalled;
-	bool iIsLocal;
 	bool iMarkedForOperation;
 	packageStatus iPkgStatus;
-	QString iVersionAvailable;
-	QString iVersionInstalled;
+	QString iVersion;
 	QString iDescriptionShort;
 	QString iDescriptionLong;
 	QString iSection;
 	int iSize;
 	int iInstalledSize;
 	operation iMarkedOperation;
+	QString iFileNameFull;
+	QDateTime iDate;
+	QList<Repository*> iRepositories;
 
 	QByteArray iIconData;
-	QPixmap iIcon;
+	QPixmap* iIcon;
 
 };
 
