@@ -26,6 +26,8 @@
 #include "dimmer.h"
 #include "packageselector.h"
 #include "help.h"
+#include "aaptinterface.h"
+#include "logview.h"
 
 PackageListWidgetItem::PackageListWidgetItem(Package* p_, QString name_) : QListWidgetItem(name_)
 {
@@ -36,12 +38,13 @@ PackageListWidgetItem::PackageListWidgetItem(Package* p_, QString name_) : QList
 void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
 	QString name = index.data(UserRoleName).toString();
-	QString verInst = index.data(UserRoleVerInst).toString();
-	QString verAvail = index.data(UserRoleVerAvail).toString();
+	QString version = index.data(UserRoleVersion).toString();
 	QString desc = index.data(UserRoleDescShort).toString();
 	Package::operation marked = (Package::operation) index.data(UserRoleMarked).toInt();
 	bool installed = index.data(UserRoleInstalled).toBool();
 	bool upgradeable = index.data(UserRoleUpgradeable).toBool();
+	int statfilter = index.data(UserRoleCurrentStatFilter).toInt();
+	QString upg_version = index.data(UserRoleAvailVersion).toString();
 
 	painter->save();
 	QRect r = option.rect;
@@ -54,24 +57,8 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 	gradientBase.setColorAt(0, base);
 	gradientBase.setColorAt(1, base2);
 
-	/*
-	QLinearGradient gradientHilight(r.topLeft(), r.bottomLeft());
-	QColor hl = option.palette.color(QPalette::Highlight);
-	QColor hl2 = hl;
-	hl.setRgb( hl.red()+10, hl.green()+10, hl.blue()+10 );
-	hl2.setRgb( hl2.red()-30, hl2.green()-30, hl2.blue()-30 );
-	gradientHilight.setColorAt(0, hl);
-	gradientHilight.setColorAt(1, hl2);
-
-	if( marked != Package::PkgOpNone ) {
-		painter->fillRect(r, gradientHilight);
-	} else {
-		painter->fillRect(r, gradientBase);
-	}
-	*/
 	painter->fillRect(r, gradientBase);
 
-	//painter->drawLine(QPoint(r.left(),r.top()), QPoint(r.right(),r.top()));
 	painter->drawLine(QPoint(r.left(),r.bottom()), QPoint(r.right(),r.bottom()));
 
 	QPixmap icon = index.data(Qt::DecorationRole).value<QPixmap>();
@@ -100,18 +87,19 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 		statusicon = iIconPkgRemove;
 	}
 
+	QString showVer = "";
+	if( upgradeable && statfilter==Package::PkgStatUpgradeable )
+		showVer = upg_version;
+	else
+		showVer = version;
+
 	int ver_w = 0;
 	if( QApplication::desktop()->width() > QApplication::desktop()->height() )
 	{
 		r = option.rect;
 		r.setRight( r.right()-statusicon.width()-4 );
-		if( installed && !upgradeable ) {
-			painter->drawText(r, Qt::AlignTop|Qt::AlignRight, verInst, &r);
-			ver_w = r.width();
-		} else {
-			painter->drawText(r, Qt::AlignTop|Qt::AlignRight, verAvail, &r);
-			ver_w = r.width();
-		}
+		painter->drawText(r, Qt::AlignTop|Qt::AlignRight, showVer, &r);
+		ver_w = r.width();
 	}
 
 	r = option.rect;
@@ -154,7 +142,7 @@ QSize ListItemDelegate::sizeHint(const QStyleOptionViewItem&, const QModelIndex&
 
 PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::PackageView)
 {
-	iMainWindow = static_cast<MainWindow*>(parent);
+	iMainWindow = dynamic_cast<MainWindow*>(parent);
 	ui->setupUi(this);
 #ifdef Q_WS_MAEMO_5
 	this->setAttribute(Qt::WA_Maemo5StackedWindow);
@@ -166,7 +154,7 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 
 	iCatFilterLabels
 			<< tr("All marked packages")	// a special case
-			<< tr("All user categories")	// special case: don't show "hidden" category
+			<< tr("All user categories")
 			<< tr("\tDesktop")
 			<< tr("\tEducation")
 			<< tr("\tGames")
@@ -210,12 +198,13 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 			<< tr("Installed");
 
 	iSelectedStatFilter = Package::PkgStatUnknown;
+	iSortOrder = SortAlpha;
 
 	iDimmer = new dimmer(this);
 
 	ui->searchBar->hide();
 
-	iListCoverLabel = new QLabel("No packages",ui->listWidget);
+	iListCoverLabel = new QLabel(ui->listWidget);
 	iListCoverLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 	iListCoverLabel->setAlignment(Qt::AlignCenter);
 
@@ -243,13 +232,53 @@ void PackageView::resizeEvent(QResizeEvent* event)
 	QWidget::resizeEvent(event);
 }
 
-void PackageView::openWin(MainWindow::operation op_)
+bool PackageView::doFilterCategory(Package* pkg)
+{
+	if( pkg->section()=="user/hidden" )
+		return false;
+
+	if( iSelectedCatFilter==CatFilterAllMarked ) {
+		if( pkg->isMarkedForOperation() )
+			return true;
+		else
+			return false;
+	}
+	if( pkg->section().startsWith( iCatFilterStrings.at(iSelectedCatFilter) ) )
+		return true;
+
+	return false;
+}
+
+QString PackageView::generateSortString(Package* pkg)
+{
+	// default sort order is alphabetical by name
+	QString sortstr = pkg->displayName();
+
+	if( iSortOrder==SortDateDesc )
+		sortstr = pkg->date().toString("yyyy-MM-dd hh:mm");
+	if( iSortOrder==SortSizeDesc ) {
+		if( pkg->isInstalled() )
+			sortstr = QString("%1").arg(pkg->installedSize()*1024, 12 );
+		else
+			sortstr = QString("%1").arg(pkg->size(), 12 );
+	}
+
+	//qDebug() << sortstr;
+
+	return sortstr;
+}
+
+void PackageView::openWin()
 {
-	iOperation = op_;
-	iListCoverLabel->hide();
 	ui->listWidget->clear();
 	ui->listWidget->setSortingEnabled(true);
 
+	if( iSortOrder==SortDateDesc || iSortOrder==SortSizeDesc )
+		ui->listWidget->sortItems(Qt::DescendingOrder);
+	else
+		ui->listWidget->sortItems(Qt::AscendingOrder);
+	ui->btn_Sort->hide();
+
 	delete ui->listWidget->itemDelegate();
 	ListItemDelegate* delegate = new ListItemDelegate(ui->listWidget);
 	delegate->loadIcons();
@@ -257,84 +286,51 @@ void PackageView::openWin(MainWindow::operation op_)
 
 	if( !ui->searchBar->isVisible() )
 	{
-		QHashIterator<QString, Package*> i( *iAptInterface->packages() );
-		while (i.hasNext())
+		if( iSelectedStatFilter == Package::PkgStatNotInstalled || iSelectedStatFilter == Package::PkgStatUnknown ||
+			iSelectedCatFilter == CatFilterAllMarked )
 		{
-			i.next();
-
-			QString name = i.value()->name();
-			if( i.value()->maemoDisplayName()!="" )
-				name = i.value()->maemoDisplayName();
-			QString n( name.at(0) );
-			n = n.toUpper();
-			name.replace(0,1,n);
-			bool added = false;
-
-			if( iSelectedStatFilter == Package::PkgStatNotInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+			QHashIterator<QString, Package*> i( *iAptInterface->packagesAvailable() );
+			while (i.hasNext())
 			{
-				if( !i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
-					&& !i.value()->isLocal() )
-					{
-					if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
-						addListItem(i.value(), name);
-						added = true;
-					}
-					else if( iSelectedCatFilter!=CatFilterAllUser ) {
-						addListItem(i.value(), name);
-						added = true;
-					}
-				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-					addListItem(i.value(), name);
-					added = true;
-				}
+				i.next();
+				Package* inst = iAptInterface->packagesInstalled()->value(i.value()->name(),0);
+				if( doFilterCategory(i.value()) && !inst )
+					addListItem(i.value(), generateSortString(i.value()));
 			}
-			if( iSelectedStatFilter == Package::PkgStatInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+		}
+		if( iSelectedStatFilter == Package::PkgStatInstalled || iSelectedStatFilter == Package::PkgStatUpgradeable ||
+			iSelectedStatFilter == Package::PkgStatUnknown || iSelectedCatFilter == CatFilterAllMarked )
+		{
+			QHashIterator<QString, Package*> i( *iAptInterface->packagesInstalled() );
+			while (i.hasNext())
 			{
-				if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter)) )
-				{
-					addListItem(i.value(), name);
-					added = true;
-				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-					addListItem(i.value(), name);
-					added = true;
-				}
-			}
-			if( iSelectedStatFilter == Package::PkgStatUpgradeable || iSelectedStatFilter == Package::PkgStatUnknown && !added)
-			{
-				if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->versionAvailable()!=i.value()->versionInstalled()
-					&& i.value()->versionAvailable()!="" && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
-					&& !i.value()->isLocal() )
-					{
-					if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
-						addListItem(i.value(), name);
-						added = true;
-					}
-					else if( iSelectedCatFilter!=CatFilterAllUser ) {
-						addListItem(i.value(), name);
-						added = true;
-					}
-				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-					addListItem(i.value(), name);
-					added = true;
+				i.next();
+				if( iSelectedStatFilter == Package::PkgStatUpgradeable ) {
+					if( i.value()->isUpgradeable() && doFilterCategory(i.value()) )
+						addListItem(i.value(), generateSortString(i.value()));
+				} else {
+					if( doFilterCategory(i.value()) )
+						addListItem(i.value(), generateSortString(i.value()));
 				}
 			}
 		}
 	} else {
 		for( int j=0; j<iSearchResults.count(); j++ )
 		{
-			QString name = iSearchResults.at(j)->name();
-			if( iSearchResults.at(j)->maemoDisplayName()!="" )
-				name = iSearchResults.at(j)->maemoDisplayName();
-			QString n( name.at(0) );
-			n = n.toUpper();
-			name.replace(0,1,n);
-
-			addListItem(iSearchResults.at(j), name);
+			addListItem(iSearchResults.at(j), generateSortString( iSearchResults.at(j) ));
 		}
 	}
 
+	iListCoverLabel->hide();
+	iListCoverLabel->setAutoFillBackground(false);
+
 	updateLabel();
 
+	if( iSelectedStatFilter==Package::PkgStatUpgradeable && ui->listWidget->count()>0 )
+		ui->actionUpgrade_all->setVisible(true);
+	else
+		ui->actionUpgrade_all->setVisible(false);
+
 	show();
 
 	if( !ui->searchBar->isVisible() ) {
@@ -348,6 +344,7 @@ void PackageView::openWin(MainWindow::operation op_)
 	if( ui->listWidget->count() == 0 )
 	{
 		iListCoverLabel->setGeometry( ui->listWidget->rect() );
+		iListCoverLabel->setText("No packages");
 		iListCoverLabel->show();
 	}
 }
@@ -366,7 +363,8 @@ void PackageView::addListItem(Package* pkg_, QString listname_)
 {
 	PackageListWidgetItem* p = new PackageListWidgetItem( pkg_, listname_ );
 
-	if( pkg_ != 0 ) {
+	if( pkg_ != 0 )
+	{
 		QString name = pkg_->name();
 		if( pkg_->maemoDisplayName()!="" )
 			name = pkg_->maemoDisplayName();
@@ -375,13 +373,17 @@ void PackageView::addListItem(Package* pkg_, QString listname_)
 		p->setData(UserRoleName, listname_);
 	}
 
-	if( pkg_ != 0 ) {
+	if( pkg_ != 0 )
+	{
 		p->setData(UserRoleDescShort, pkg_->descShort());
-		p->setData(UserRoleVerAvail, pkg_->versionAvailable());
-		p->setData(UserRoleVerInst, pkg_->versionInstalled());
+		p->setData(UserRoleVersion, pkg_->version());
 		p->setData(UserRoleMarked, (int)pkg_->markedOperation());
 		p->setData(UserRoleInstalled, pkg_->isInstalled());
 		p->setData(UserRoleUpgradeable, pkg_->isUpgradeable());
+		p->setData(UserRoleAvailVersion, pkg_->upgradeableVersion());
+		p->setData(UserRoleCurrentStatFilter, (int)iSelectedStatFilter);
+
+		//qDebug()<<pkg_->name();
 
 		pkg_->convertIcon();
 		p->setData(Qt::DecorationRole, *pkg_->icon());
@@ -391,27 +393,40 @@ void PackageView::addListItem(Package* pkg_, QString listname_)
 
 void PackageView::closeEvent(QCloseEvent *event)
 {
-	if( iDimmer->busy() ) {
+	if( !iAptInterface ) {
+		resetWindow();
+		event->accept();
+	}
+
+	if( iDimmer->busy() )
+	{
+		iAptInterface->cancel();
 		event->ignore();
 		return;
 	}
 
-	if( iAptInterface->selectedCount() == 0 )
+#ifdef Q_WS_MAEMO_5
+	if( iAptInterface->numSelectedPackages() == 0 )
 	{		
 		resetWindow();
 		event->accept();
 	} else {
 		QString c;
-		c.setNum( iAptInterface->selectedCount() );
+		c.setNum( iAptInterface->numSelectedPackages() );
 		ConfirmDialog d(true, this);
-		d.setText(tr("Returning to main menu"), tr("Clear ") + c + tr(" package selection(s) and lose all the pending changes?"));
+		d.setText("Returning to main menu", QString("Clear %1 package selection(s) and lose all the pending changes?").arg(iAptInterface->numSelectedPackages()));
 		if( d.exec() ) {
 			resetWindow();
+			QWidget::closeEvent(event);
 			event->accept();
 		} else {
 			event->ignore();
 		}
 	}
+#else	// for simulator
+	resetWindow();
+	event->accept();
+#endif
 }
 
 void PackageView::changeEvent(QEvent *e)
@@ -428,64 +443,75 @@ void PackageView::changeEvent(QEvent *e)
 
 void PackageView::on_btn_Commit_clicked()
 {
-	/*
-	QStringList pkgnames_inst, pkgnames_remv;
+	QStringList pkgnames;
 
-	QHashIterator<QString, Package*> i( *iAptInterface->packages() );
+	QHashIterator<QString, Package*> i( *iAptInterface->packagesAvailable() );
 	while (i.hasNext())
 	{
 		i.next();
+
 		if( i.value()->markedOperation() == Package::PkgOpInstallUpgrade )
-			pkgnames_inst << i.value()->name();
-		if( i.value()->markedOperation() == Package::PkgOpRemove )
-			pkgnames_remv << i.value()->name();
+			pkgnames << i.value()->name();
+		if( i.value()->markedOperation() == Package::PkgOpRemove ) {
+			qDebug() << "warning: trying to add package marked from the wrong list";
+			//pkgnames << i.value()->name() + "-";
+		}
 	}
 
-	std::cout << "INSTALL/UPGRADE: " << std::endl;
-	for( int x=0; x<pkgnames_inst.count(); x++ )
-		std::cout << pkgnames_inst.at(x).toStdString() << " ";
-	std::cout << std::endl;
+	QHashIterator<QString, Package*> r( *iAptInterface->packagesInstalled() );
+	while (r.hasNext())
+	{
+		r.next();
 
-	std::cout << "REMOVE: " << std::endl;
-	for( int x=0; x<pkgnames_remv.count(); x++ )
-		std::cout << pkgnames_remv.at(x).toStdString() << " ";
-	std::cout << std::endl;
-	*/
+		if( r.value()->markedOperation() == Package::PkgOpInstallUpgrade )
+			pkgnames << r.value()->name();
+		if( r.value()->markedOperation() == Package::PkgOpRemove )
+			pkgnames << r.value()->name() + "-";
+	}
 
-	iAptInterface->simulateInstall();
+	iMainWindow->busyDialog(true, "Operation in progress", "Reading dependencies");
+
+	iAptInterface->setProcessPackages(pkgnames);
+	iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetSimulate);
+	iAptInterface->run(iDimmer);
 }
 
 void PackageView::on_actionClear_selections_triggered()
 {
 	QString c;
-	c.setNum( iAptInterface->selectedCount() );
+	c.setNum( iAptInterface->numSelectedPackages() );
 	ConfirmDialog d(true, this);
 	d.setText(tr("Confirmation"), tr("Clear ") + c + tr(" package selection(s) and lose all the pending changes?"));
 	if( d.exec() )
 	{
 		clearSelections();
-		openWin(iOperation);
+		openWin();
 	}
 }
 
 void PackageView::clearSelections()
 {
-	QHashIterator<QString, Package*> i( *iAptInterface->packages() );
+	QHashIterator<QString, Package*> i( *iAptInterface->packagesInstalled() );
 	while (i.hasNext())
 	{
 		i.next();
 		i.value()->setMarkedForOperation(Package::PkgOpNone);
 	}
-
-	iAptInterface->resetSelectedCount();
+	QHashIterator<QString, Package*> a( *iAptInterface->packagesAvailable() );
+	while (a.hasNext())
+	{
+		a.next();
+		a.value()->setMarkedForOperation(Package::PkgOpNone);
+	}
+	iAptInterface->setNumSelectedPackages(0);
 }
 
 void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 {
-	PackageSelector s(static_cast<PackageListWidgetItem*>(item)->package(), this);
+	PackageSelector s(dynamic_cast<PackageListWidgetItem*>(item)->package(), this);
 	s.exec();
 
-	static_cast<PackageListWidgetItem*>(item)->package()->setMarkedForOperation( s.selectedOperation() );
+	dynamic_cast<PackageListWidgetItem*>(item)->package()->setMarkedForOperation( s.selectedOperation() );
 	item->setData(UserRoleMarked, (int)s.selectedOperation());
 
 	updateLabel();
@@ -494,17 +520,17 @@ void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 void PackageView::updateLabel()
 {
 	QString s;
-	s.setNum( iAptInterface->selectedCount() );
+	s.setNum( iAptInterface->numSelectedPackages() );
 	QString s2;
 	s2.setNum( ui->listWidget->count() );
 	QString statlabel = iStatFilterLabels.at(iSelectedStatFilter);
 	if( iSelectedCatFilter == CatFilterAllMarked )
 		statlabel = "All";
-	ui->label->setText("<font size=\"-2\"><b>" + s + "</b> package(s) marked for operation<br>"
+	ui->label->setText("<font size=\"-2\"><b>" + s + "</b> package(s) marked<br>"
 					   + "Showing: <b>" + statlabel + "</b><br>"
 					   + "Filter: " + iCatFilterLabels.at(iSelectedCatFilter) + " - " + s2 + " package(s)</font>");
 
-	if( iAptInterface->selectedCount()==0 ) {
+	if( iAptInterface->numSelectedPackages()==0 ) {
 		ui->btn_Commit->setEnabled(false);
 		ui->actionClear_selections->setVisible(false);
 	} else {
@@ -529,8 +555,14 @@ void PackageView::on_btn_CategoryFilter_clicked()
 		ui->btn_StatusFilter->setEnabled(true);
 	}
 
-	if( s )
-		openWin(iOperation);
+	if( s ) {
+		iListCoverLabel->setText("Loading...");
+		iListCoverLabel->setAutoFillBackground(true);
+		iListCoverLabel->show();
+		QApplication::processEvents();
+
+		openWin();
+	}
 }
 
 void PackageView::setStatFilter(Package::packageStatus f_)
@@ -547,7 +579,13 @@ void PackageView::on_btn_StatusFilter_clicked()
 
 	if( s ) {
 		iSelectedStatFilter = (Package::packageStatus)f.selection();
-		openWin(iOperation);
+
+		iListCoverLabel->setText("Loading...");
+		iListCoverLabel->setAutoFillBackground(true);
+		iListCoverLabel->show();
+		QApplication::processEvents();
+
+		openWin();
 	}
 }
 
@@ -556,10 +594,9 @@ void PackageView::resetWindow()
 	iSelectedCatFilter = iDefaultCatFilter;
 	ui->btn_StatusFilter->setEnabled(true);
 
-	releaseKeyboard();
 	clearSelections();
-	if( iMainWindow )
-		iMainWindow->setOperation(MainWindow::OpNone);
+	//if( iMainWindow )
+	//	iMainWindow->setOperation(MainWindow::OpNone);
 
 	clearSearch();
 }
@@ -573,7 +610,13 @@ void PackageView::on_actionHelp_triggered()
 void PackageView::on_btn_searchClose_clicked()
 {
 	clearSearch();
-	openWin(iOperation);
+
+	iListCoverLabel->setText("Loading...");
+	iListCoverLabel->setAutoFillBackground(true);
+	iListCoverLabel->show();
+	QApplication::processEvents();
+
+	openWin();
 }
 
 void PackageView::clearSearch()
@@ -592,7 +635,7 @@ void PackageView::on_actionSearch_triggered()
 		return;
 
 	ui->listWidget->setFocusPolicy(Qt::NoFocus);
-	ui->searchLabel->setText( QString("%0 results").arg(ui->listWidget->count()) );
+	ui->searchLabel->setText( QString("%1 results").arg(ui->listWidget->count()) );
 	ui->toolBar->hide();
 	ui->searchBar->show();
 	ui->lineEdit->setFocus();
@@ -608,8 +651,13 @@ void PackageView::on_lineEdit_textEdited(QString text)
 
 	if( iPrevSearchText.length() > text.length() )
 	{
+		iListCoverLabel->setText("Loading...");
+		iListCoverLabel->setAutoFillBackground(true);
+		iListCoverLabel->show();
+		QApplication::processEvents();
+
 		ui->searchBar->hide();
-		openWin(iOperation);
+		openWin();
 		ui->searchBar->show();
 		ui->lineEdit->setFocus();
 	}
@@ -621,14 +669,14 @@ void PackageView::on_lineEdit_textEdited(QString text)
 	else
 		results = ui->listWidget->findItems( text, Qt::MatchContains );
 
-	ui->searchLabel->setText( QString("%0 results").arg(results.count()) );
+	ui->searchLabel->setText( QString("%1 results").arg(results.count()) );
 
 	iSearchResults.clear();
 	for( int i=0; i<results.count(); i++ )
 	{
-		iSearchResults.append( static_cast<PackageListWidgetItem*>(results.at(i))->package() );
+		iSearchResults.append( dynamic_cast<PackageListWidgetItem*>(results.at(i))->package() );
 	}
-	openWin(iOperation);
+	openWin();
 }
 
 void PackageView::setSearchText(QString text)
@@ -644,8 +692,8 @@ KeyEventGrabber::KeyEventGrabber(QObject* parent) : QObject(parent)
 bool KeyEventGrabber::eventFilter(QObject *obj, QEvent *event)
 {
 	if( event->type() == QEvent::KeyPress ) {
-		QString text = static_cast<QKeyEvent*>(event)->text();
-		int key = static_cast<QKeyEvent*>(event)->key();
+		QString text = dynamic_cast<QKeyEvent*>(event)->text();
+		int key = dynamic_cast<QKeyEvent*>(event)->key();
 		if( (text.trimmed() != "" || text==" ") && key!=Qt::Key_Backspace ) {
 			dynamic_cast<PackageView*>(this->parent())->on_actionSearch_triggered();
 			dynamic_cast<PackageView*>(this->parent())->setSearchText( text );
@@ -654,3 +702,35 @@ bool KeyEventGrabber::eventFilter(QObject *obj, QEvent *event)
 	}
 	return QObject::eventFilter(obj, event);
 }
+
+void PackageView::on_actionUpgrade_all_triggered()
+{
+	for( int i=0; i<ui->listWidget->count(); i++ )
+	{
+		Package* pkg = dynamic_cast<PackageListWidgetItem*>(ui->listWidget->item(i))->package();
+		if( pkg->isUpgradeable() ) {
+			pkg->setMarkedForOperation( Package::PkgOpInstallUpgrade );
+			ui->listWidget->item(i)->setData(UserRoleMarked, (int)Package::PkgOpInstallUpgrade);
+		}
+	}
+	updateLabel();
+}
+
+void PackageView::on_btn_Sort_clicked()
+{
+	if( iSortOrder==SortAlpha )
+		iSortOrder = SortDateDesc;
+	else if( iSortOrder==SortDateDesc )
+		iSortOrder = SortSizeDesc;
+	else if( iSortOrder==SortSizeDesc )
+		iSortOrder = SortAlpha;
+
+	openWin();
+}
+
+void PackageView::on_actionView_log_triggered()
+{
+	QByteArray log = iAptInterface->readLogFile();
+	LogView l(log, this);
+	l.exec();
+}
