@@ -35,8 +35,9 @@ class Package;
 class dimmer;
 class KeyEventGrabber;
 
-enum UserRoles { UserRoleName=Qt::UserRole, UserRoleVerInst, UserRoleVerAvail, UserRoleDescShort,
-				 UserRoleMarked, UserRoleInstalled, UserRoleUpgradeable };
+enum UserRoles { UserRoleName=Qt::UserRole, UserRoleVersion, UserRoleDescShort,
+				 UserRoleMarked, UserRoleInstalled, UserRoleUpgradeable, UserRoleAvailVersion,
+				 UserRoleCurrentStatFilter };
 
 
 class PackageListWidgetItem : public QListWidgetItem
@@ -75,13 +76,15 @@ class PackageView : public QMainWindow
 {
     Q_OBJECT
 
+	enum sortOrder { SortAlpha, SortSizeDesc, SortDateDesc };
+
 public:
     explicit PackageView(QWidget *parent = 0);
     ~PackageView();
-	void openWin(MainWindow::operation op_);
+	void openWin();
 	void resizeEvent(QResizeEvent* event);
-	void setAptInterface(AptInterface* a_) { iAptInterface=a_; }	
-	dimmer* dim() { return iDimmer; }
+	void setAptInterface(AAptInterface* a_) { iAptInterface=a_; }
+	dimmer* mydimmer() { return iDimmer; }
 	void disableMenu();
 	void enableMenu();
 	void setStatFilter(Package::packageStatus f_);
@@ -90,11 +93,21 @@ public:
 
 #ifdef Q_WS_MAEMO_5		// workaround for buggy Qt autorotation
 	void show() {
-		QRect r = static_cast<QMainWindow*>(this->parent())->rect();
-		if(r.width() < r.height())
-			this->setAttribute(Qt::WA_Maemo5PortraitOrientation);
-		else
-			this->setAttribute(Qt::WA_Maemo5AutoOrientation);
+		if( !isVisible() ) {
+			QRect r = dynamic_cast<QMainWindow*>(this->parent())->rect();
+			if(r.width() < r.height()) {
+				this->setAttribute(Qt::WA_Maemo5PortraitOrientation);
+			}
+			else {
+				if( dynamic_cast<QMainWindow*>(this->parent())->testAttribute( Qt::WA_Maemo5AutoOrientation ) ) {
+					this->setAttribute( Qt::WA_Maemo5AutoOrientation );
+				} else {
+					this->setAttribute( Qt::WA_Maemo5LandscapeOrientation );
+				}
+				this->setAttribute(Qt::WA_Maemo5StackedWindow);
+				this->setWindowFlags(Qt::Window);
+			}
+		}
 		QMainWindow::show();
 	}
 #endif
@@ -105,11 +118,13 @@ public slots:
 
 protected:
     void changeEvent(QEvent *e);
-	AptInterface* iAptInterface;
-	MainWindow::operation iOperation;
+	AAptInterface* iAptInterface;
+	//MainWindow::operation iOperation;
 	void closeEvent(QCloseEvent *event);
 
 private:
+	bool doFilterCategory(Package* pkg);
+	QString generateSortString(Package* pkg);
 	void addListItem(Package* pkg_, QString listname_);
 	void updateLabel(void);
 	void resetWindow();
@@ -130,13 +145,17 @@ private:
 
 	QStringList iStatFilterLabels;
 	int iSelectedStatFilter;
+	sortOrder iSortOrder;
 
 	QList<Package*> iSearchResults;
 	QString iPrevSearchText;
 	KeyEventGrabber* iKeyFilter;
 
 private slots:
-	void on_lineEdit_textEdited(QString );
+	void on_actionView_log_triggered();
+ void on_btn_Sort_clicked();
+ void on_actionUpgrade_all_triggered();
+ void on_lineEdit_textEdited(QString );
 	void on_btn_searchClose_clicked();
 	void on_actionHelp_triggered();
 	void on_btn_StatusFilter_clicked();
