@@ -19,6 +19,7 @@
 
 #include "settings.h"
 #include "ui_settings.h"
+#include "mainwindow.h"
 
 Settings::Settings(QWidget *parent) :
     QDialog(parent),
@@ -50,12 +51,26 @@ void Settings::changeEvent(QEvent *e)
 
 void Settings::openWin()
 {
+	ui->checkBox_autorotation->setChecked( iQSettings->value("disable_autorotation", false).toBool() );
+	ui->checkBox_autoremove->setChecked( iQSettings->value("enable_autoremove", false).toBool() );
+	ui->checkBox_autoclean->setChecked( iQSettings->value("enable_autoclean", false).toBool() );
 
-	show();
+	open();
 }
 
 void Settings::on_btn_OK_clicked()
 {
+	iQSettings->setValue("disable_autorotation", ui->checkBox_autorotation->isChecked() );
+	iQSettings->setValue("enable_autoremove", ui->checkBox_autoremove->isChecked() );
+	iQSettings->setValue("enable_autoclean", ui->checkBox_autoclean->isChecked() );
+
+#ifdef Q_WS_MAEMO_5
+	if( !iQSettings->value("disable_autorotation",false).toBool() ) {
+		dynamic_cast<MainWindow*>(this->parent())->setAttribute(Qt::WA_Maemo5AutoOrientation);
+	} else {
+		dynamic_cast<MainWindow*>(this->parent())->setAttribute(Qt::WA_Maemo5LandscapeOrientation);
+	}
+#endif
 
 	accept();
 }
